package org.ntnu.pythia.engine;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilePermission;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.security.AccessControlException;
import java.security.AccessController;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.ntnu.pythia.engine.components.OutputSplitter;
import org.ntnu.pythia.python.Globals;

import uk.co.flamingpenguin.jewel.cli.ArgumentValidationException;
import uk.co.flamingpenguin.jewel.cli.CliFactory;

public class App {
	/** Global Python variables */	
	public static Globals globals;
	/** Output from Python */
	public static final OutputSplitter output = new OutputSplitter();
	/** Command-line options for Pythia */
	public static Options options;
	
	
	private final static String libraryPath;
	
	private static String getOS() {
		String os_name = System.getProperty("os.name").toLowerCase();
		if (os_name.contains("windows")) return "windows";
		if (os_name.contains("linux")) return "linux";
		if (os_name.contains("macosx")) return "macosx";
		if (os_name.contains("solaris")) return "solaris";
		return null;
	}
	
	private static String getArch() {
		String arch = System.getProperty("os.arch");
		if (arch.endsWith("64")) return "64";
		else return "x86";
	}
	
	private static void copyLibraries(String os, String arch, File tempDir) {
		byte[] buf = new byte[1024];
		int len;
		try {
			URL dirURL = App.class.getResource("/" + os);
			JarFile jar = new JarFile(URLDecoder.decode((dirURL.getFile().substring(5, dirURL.getPath().indexOf('!'))), "UTF-8"));
			//JarFile jar = new JarFile(dirURL.getFile().substring(5, dirURL.getPath().indexOf('!')));
			Enumeration<JarEntry> entries = jar.entries();
			while(entries.hasMoreElements()) {
				String name = entries.nextElement().getName();
				if (name.startsWith(os)) {
					// Extract filename and remove the first slash 
					String filename = name.substring(os.length() + 1);
					// Ignore sub-folders and current folder
					if (filename.contains("/") || filename.trim().length() == 0) continue;
					// Ignore 64-bit libraries on 32-bit machine
					//if (arch.equals("x86") && filename.contains("64")) continue;
					
					InputStream in = App.class.getResourceAsStream("/" + name);
					File output = new File(tempDir, filename);
					if (!output.createNewFile()) throw new IOException("Cannot create file");
					
					System.out.println("copying: " + filename);
					
					OutputStream out = new FileOutputStream(output);
					while ((len = in.read(buf)) > 0)
						out.write(buf, 0, len);
					in.close();
					out.close();
					output.deleteOnExit();
				}
			}
		} catch (FileNotFoundException e) {
			System.err.println("Library not found!");
			e.printStackTrace();
			System.exit(1);
		} catch (IOException e) {
			System.err.println("Error extracting library: "+e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	static {
		String os = getOS();
		String arch = getArch();
		File tempDir = null;
		if (os == null) {
			System.err.println("Unsupported OS");
			System.exit(1);
		}
		try {
			tempDir = File.createTempFile("pythia", "native");
		} catch (IOException e) {
			System.err.println("Cannot create temporary folder!");
			System.exit(1);
		}
		
		tempDir.delete();
		tempDir.mkdir();
		tempDir.deleteOnExit();
		libraryPath = tempDir.getAbsolutePath();
		copyLibraries(os, arch, tempDir);
		
		System.setProperty("org.lwjgl.librarypath", libraryPath);
		
		
		System.out.printf("OS: %s\nTemporary folder: %s\n", os, libraryPath);
	}
	
	/**
	 * Main program entry point.
	 * @param args
	 * @throws ArgumentValidationException if arguments could not be parsed correctly.
	 */
    public static void main( String[] args ) {
    	try{
    	options = CliFactory.parseArguments(Options.class, args);
    	}catch (ArgumentValidationException e) {
    		System.out.println(e.getMessage());
    		System.exit(0);
		}
    	
    	globals = Globals.readGlobals();
    	
    	GameWindow window = ResourceFactory.getGameWindow();
    	    	
    	Game game = ResourceFactory.getGame();
    	window.setGame(game);
    	try{
    		window.startRender();
    	}catch (Exception e) {
    		if(options.isDebug()){
    			e.printStackTrace();
    			System.exit(1);
    		}else{
    			ResourceFactory.getGameWorld().messageBox("Terminal error. Someone cocked up. " +e.getMessage());
    			System.exit(1);
    		}
		}
    }
}
