package org.ntnu.pythia.engine;

import org.ntnu.pythia.game.GameObject;
import org.ntnu.pythia.game.level.Level;
import org.ntnu.pythia.python.unit.Tester;

/**
 * This class is used for the Game Objects to interact with the Game World
 * @author egr
 *
 */
public class GameWorld {
	private final Game game;
	private static final Tester tester  = Tester.getTester();
	
	public GameWorld(Game game) {
		this.game = game;
		tester.setGame(game);
	}
	
	/**
	 * A game object can show a message through this method
	 * @param text
	 */
	public void messageBox(String text) {
		game.showMessageBox(text);
	}
	
	public void say(GameObject go, String text) {
		game.showMessage(text, 3000, go, go.getHeight()/2);
	}
	
	/**
	 * A game object can do a status box update through this method
	 * @param text
	 */
	public void statusUpdate(String text) {
		game.statusUPdate(text);
	}
	
	public void showObjectives(){
		tester.showRemainingObjectives();	
	}
	
	public Tester getTester(){
		return tester;	
	}
	
	public void loadLevel(String level){
		game.loadLevel(level);
	}
	
	public void updateTests(Level level){
		tester.loadTests(level);
	}

	/**
	 * Removes an object from the game universes (rendering and collision universes)
	 * @param o
	 */
	public void removeGameObject(GameObject o) {
		game.removeGameObject(o);
	}

	public void addGameObject(GameObject obj) {
		game.addGameObject(obj);
	}
	
	public void setRenderCollision(boolean value) {
		ResourceFactory.setRenderCollision(value);
	}
	
	public void showConsole(String text) {
		game.showConsole(text);
	}
	
	public void showConsole() {
		game.showConsole();
	}
	
	public void showEditor() {
		game.showEditor();
	}
	
	public GameObject getObjectByName(String name){
		return game.getObjectByName(name);
	}
	
	public Camera getCamera() {
		return game.getCamera();
	}
	
}
