package org.ntnu.pythia.engine.components;

import org.lwjgl.opengl.GL11;
import org.ntnu.pythia.engine.Game;

import de.matthiasmann.twl.BoxLayout;
import de.matthiasmann.twl.Button;
import de.matthiasmann.twl.Event;
import de.matthiasmann.twl.GUI;
import de.matthiasmann.twl.Label;
import de.matthiasmann.twl.PopupWindow;
import de.matthiasmann.twl.Widget;
import de.matthiasmann.twl.BoxLayout.Direction;

public class GameMenu extends PopupWindow {
	class QuestionPopup extends PopupWindow {
		public QuestionPopup(Widget owner, String question, Runnable callback) {
			super(owner);
			setTheme("fixedframe-title");
			
			BoxLayout layout1 = new BoxLayout(Direction.VERTICAL);
			BoxLayout layout2 = new BoxLayout(Direction.HORIZONTAL);
			layout1.setTheme("/boxlayout");
			layout2.setTheme("/boxlayout");
			
			Label label = new Label(question);
			label.setTheme("/label");
			
			Button buttonOk = new Button("OK (Enter)");
			Button buttonCancel = new Button("Cancel (ESC)");
			buttonOk.addCallback(callback);
			buttonOk.addCallback(new Runnable() {
				public void run() { closePopup(); }
			});
			buttonCancel.addCallback(new Runnable() {
				public void run() { closePopup(); }
			});
			buttonOk.setTheme("/button");
			buttonCancel.setTheme("/button");
			
			layout2.add(buttonOk);
			layout2.add(buttonCancel);
			
			layout1.add(label);
			layout1.add(layout2);
			
			add(layout1);
		}
		
		@Override
		protected boolean handleEventPopup(Event evt) {
			if (evt.getType() == Event.Type.KEY_PRESSED) {
				switch (evt.getKeyCode()) {
				case Event.KEY_LEFT:
					focusPrevChild();
					return true;
				case Event.KEY_RIGHT:
					focusNextChild();
					return true;
				default:
					break;
				}
			}
			return super.handleEventPopup(evt);
		}
		
	}
	
	Runnable quitCallback = new Runnable() {
		@Override
		public void run() {
			game.quitGame();
			closePopup();
		}
	};
	private Game game;

	public GameMenu(Widget owner, final Game game) {
		super(owner);
		this.game = game;
		
		BoxLayout layout = new BoxLayout(Direction.VERTICAL);
		layout.setTheme("/boxlayout");
		Button buttonRestart = new Button("Restart Level (F9)");
		Button button3 = new Button("Quit game (F10)");
		Label label = new Label();
		Button button4 = new Button("Resume (ESC)");
		button3.addCallback(new Runnable() {
			public void run() {
				functionQuit();
			}
		});
		button4.addCallback(new Runnable() {
			public void run() {
				closePopup();
			}
		});
		buttonRestart.addCallback(new Runnable() {
			public void run() {
				game.restartLevel();
				closePopup();
			}
		});
		label.setTheme("/label");
		button3.setTheme("/button");
		button4.setTheme("/button");
		buttonRestart.setTheme("/button");
		
		layout.add(button3);
		layout.add(buttonRestart);
		layout.add(label);
		layout.add(button4);
		setTheme("fixedframe-title");
		add(layout);
	}
	
	private void functionQuit() {
		QuestionPopup popup = new QuestionPopup(getParent(), "Are you sure you want to quit?", quitCallback);
		popup.openPopupCentered();
	}
	
	@Override
	protected boolean handleEventPopup(Event evt) {
		if (evt.getType() == Event.Type.KEY_PRESSED) {
			switch (evt.getKeyCode()) {
			case Event.KEY_F9:
				game.restartLevel();
				return true;
			case Event.KEY_UP:
				focusPrevChild();
				return true;
			case Event.KEY_DOWN:
				focusNextChild();
				return true;
			case Event.KEY_F10:
				functionQuit();
				return true;
			default:
				break;
			}
		}
		return super.handleEventPopup(evt);
	}
	
	@Override
	protected void paint(GUI gui) {
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		
		GL11.glColorMaterial(GL11.GL_FRONT, GL11.GL_AMBIENT);
		GL11.glColor4f(1, 0, 0, 0.5f);
		GL11.glRecti(0, 0, gui.getWidth(), gui.getHeight());
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		super.paint(gui);
	}

}
