package org.ntnu.pythia.engine.components;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.engine.Game;
import org.ntnu.pythia.engine.ResourceFactory;

import de.matthiasmann.twl.BoxLayout;
import de.matthiasmann.twl.BoxLayout.Direction;
import de.matthiasmann.twl.Button;
import de.matthiasmann.twl.Event;
import de.matthiasmann.twl.PopupWindow;
import de.matthiasmann.twl.Widget;

public class MainMenu extends PopupWindow {
	private final Game game;

	public MainMenu(Widget owner, final Game game) {
		super(owner);
		this.game = game;
		
		BoxLayout layout = new BoxLayout(Direction.VERTICAL);
		layout.setTheme("/boxlayout");
		Button button1 = new Button("New game (F1)");
		Button button2 = new Button("Continue (F2)");
		Button button3 = new Button("Credits  (F3)");
		Button button4 = new Button("Quit     (F4)");
		button1.addCallback(new Runnable() {
			public void run() {	functionNewGame(); }
		});
		button2.addCallback(new Runnable() {
			public void run() { functionLoadGame();	}
		});
		button3.addCallback(new Runnable(){
			public void run() { functionCredits(); }
		});
		button4.addCallback(new Runnable(){
			public void run() {	functionExit();	}
		});	
		button1.setTheme("/button");
		button2.setTheme("/button");
		button3.setTheme("/button");
		button4.setTheme("/button");
		
		if( !App.globals.isLoadable())
			button2.setEnabled(false);
		
		layout.add(button1);
		layout.add(button2);
		layout.add(button3);
		layout.add(button4);
		setTheme("fixedframe-title");
		add(layout);
	}
	
	private void functionNewGame() {
		App.globals.newPlayer("Player");//will create a new save slot. Currently we have only one. 
		game.startNewGame();
		closePopup();
	}
	private void functionLoadGame() {
		game.startNewGame();
		closePopup();
	}
	private void functionCredits() {
		ResourceFactory.getGameWorld().messageBox("Credits:");//Just a suggestion..
	}
	private void functionExit() {
		System.out.println("Quitting Pythia..");
		System.exit(0);
	}
	
	@Override
	protected boolean handleEventPopup(Event evt) {
		if (evt.getType() == Event.Type.KEY_RELEASED) {
			switch (evt.getKeyCode()) {
			case Event.KEY_F1:
				functionNewGame();
				return true;
			case Event.KEY_F2:
				functionLoadGame();
				return true;
			case Event.KEY_F3:
				functionCredits();
				return true;
			case Event.KEY_F4:
				functionExit();
				return true;
			case Event.KEY_UP:
				focusPrevChild();
				return true;
			case Event.KEY_DOWN:
				focusNextChild();
				return true;
			default:
				break;
			}
		}
		return super.handleEventPopup(evt);
	}

}
