package org.ntnu.pythia.engine.lwjgl;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.geom.Point2D;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.FloatBuffer;
import java.util.Arrays;
import java.util.List;

import org.lwjgl.BufferUtils;
import org.lwjgl.LWJGLException;
import org.lwjgl.input.Mouse;
import org.lwjgl.openal.AL;
import org.lwjgl.openal.AL10;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.opengl.TextureImpl;
import org.newdawn.slick.util.ResourceLoader;
import org.ntnu.pythia.engine.Game;
import org.ntnu.pythia.engine.GameWindow;
import org.ntnu.pythia.engine.ResourceFactory;
import org.ntnu.pythia.game.GameObject;

public class LWJGLGameWindow implements GameWindow {
	private Game mGame;
	private TextureLoader textureLoader;
	
	private int width;
	private int height;
	
	private AngelCodeFont font;
	
	TextureLoader getTextureLoader() {
		return textureLoader;
	}
	
	/** Position of the listener. */
	FloatBuffer listenerPos = (FloatBuffer)BufferUtils.createFloatBuffer(3).put(new float[] { 0.0f, 0.0f, 0.0f }).rewind();
	/** Velocity of the listener. */
	FloatBuffer listenerVel = (FloatBuffer)BufferUtils.createFloatBuffer(3).put(new float[] { 0.0f, 0.0f, 0.0f }).rewind();
	/** Orientation of the listener. (first 3 elements are "at", second 3 are "up") */
	FloatBuffer listenerOri = (FloatBuffer)BufferUtils.createFloatBuffer(6).put(new float[] { 0.0f, 0.0f, -1.0f,  0.0f, 1.0f, 0.0f }).rewind();
	
	public LWJGLGameWindow(String title, int width, int height) {
		try {
			Display.setTitle(title);
			Display.setVSyncEnabled(false);
			Display.setFullscreen(false);
			DisplayMode dm[] = org.lwjgl.util.Display.getAvailableDisplayModes(width, height, -1, -1, -1, -1, 60, 60);
			
			System.out.println("Available modes:");
			for (int i = 0; i < dm.length; i++) {
				DisplayMode d = dm[i];
				System.out.println("Mode: " + d.getWidth() + "x" + d.getHeight());
				if (d.getWidth() == width && d.getHeight() == height)
					Display.setDisplayMode(d);
			}
			//Display.setDisplayMode(dm[0]);
			
			
			intiSound();
			
			Display.create();
			
			this.height = Display.getDisplayMode().getHeight();
			this.width = Display.getDisplayMode().getWidth();
			
			Mouse.setGrabbed(false);
			
			GL11.glEnable(GL11.GL_TEXTURE_2D);
			
			GL11.glDisable(GL11.GL_DEPTH_TEST);
			
			GL11.glAlphaFunc(GL11.GL_GREATER, 0f);
			GL11.glEnable(GL11.GL_ALPHA_TEST);
			
			GL11.glMatrixMode(GL11.GL_PROJECTION);
			GL11.glLoadIdentity();
			
			GL11.glOrtho(0, width, height, 0, -1, 1);
			
			GL11.glEnable (GL11.GL_BLEND);
			GL11.glBlendFunc (GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
			
			textureLoader = new TextureLoader();
			
			Font awtFont;
			try {
				awtFont = Font.createFont(Font.TRUETYPE_FONT, ResourceLoader.getResourceAsStream("fonts/com64.ttf"));
				awtFont = awtFont.deriveFont(24f);
				font = new AngelCodeFont("com64", ResourceLoader.getResourceAsStream("fonts/com64.fnt"), ResourceLoader.getResourceAsStream("fonts/com64_0.png"));
				//font = new AngelCodeFont("xfiles", ResourceLoader.getResourceAsStream("fonts/xfiles.fnt"), ResourceLoader.getResourceAsStream("fonts/xfiles_0.png"));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (FontFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SlickException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		} catch (LWJGLException e) {
			System.err.println("Could not initialize LWJGL window: " + e.getMessage());
			e.printStackTrace(System.err);
			System.exit(1);
		}
		
	}
	
	private void intiSound() {
		try {
			AL.create();
		} catch (LWJGLException e) {
			e.printStackTrace();
			return;
		}
		AL10.alGetError();
		AL10.alListener(AL10.AL_POSITION,    listenerPos);
		AL10.alListener(AL10.AL_VELOCITY,    listenerVel);
		AL10.alListener(AL10.AL_ORIENTATION, listenerOri);
	}

	@Override
	public void startRender() {
		int fps_limit = ResourceFactory.FPS_LIMIT;
		long fps = 0;
		int frameCounter = 0;
		long frameTime = 0;
		long lastFrameTime = System.nanoTime();
		long currentFrameTime, delta = 0;
		long[] benchmark = new long[10];
		long tmp;
		while (true) {
			currentFrameTime = System.nanoTime();
			
			tmp = System.nanoTime();
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
			GL11.glMatrixMode(GL11.GL_MODELVIEW);
			GL11.glLoadIdentity();
			benchmark[7] += System.nanoTime() - tmp;
			if (mGame != null) {
				mGame.run();
				tmp = System.nanoTime();
				Point2D.Double p = mGame.getCameraPosition();
				
				List<GameObject> sprites = mGame.getSortedGameObjects();
				benchmark[1] += System.nanoTime() - tmp;

				tmp = System.nanoTime();
				GL11.glPushMatrix();
				GL11.glTranslated(p.x, p.y, 0);

				benchmark[6] += System.nanoTime() - tmp;
				tmp = System.nanoTime();
				for (GameObject o : sprites)
					o.draw();
				
				
				GL11.glPopMatrix();
				benchmark[2] += System.nanoTime() - tmp;
			}
			tmp = System.nanoTime();
			
			try {
				mGame.getGUI().update();
			} catch (StringIndexOutOfBoundsException e) {
				//System.err.println("Caught exception in gui.update()");
				//FIXME: Strange error, unknown
			} catch (ArrayIndexOutOfBoundsException e) {
				//System.err.println("Caught exception in gui.update()");
				//FIXME: Strange error, unknown
			}
			benchmark[4] += System.nanoTime() - tmp;
			
			
			GL11.glDisable(GL11.GL_TEXTURE_2D);
			
			GL11.glColorMaterial(GL11.GL_FRONT, GL11.GL_AMBIENT);
			GL11.glColor4f(1, 0, 0, mGame.transition_progress);
			GL11.glRecti(0, 0, width, height);
			
			TextureImpl.bindNone();
			GL11.glPushMatrix();
			GL11.glScalef(0.5f, 0.5f, 1f);
			font.drawString(20, 100, "fps: "+fps, Color.green);
			GL11.glPopMatrix();
			tmp = System.nanoTime();
			Display.update();
			Display.sync(fps_limit);
			benchmark[3] += System.nanoTime() - tmp;			
			
			tmp = System.nanoTime();
			if(!Display.isActive()) {
				mGame.getGUI().clearKeyboardState();
				mGame.getGUI().clearMouseState();
            }
			benchmark[5] += System.nanoTime() - tmp;
			
			
			if (Display.isCloseRequested()) {
				clearResources();
				return;
			}
			
			if (frameCounter % 100 == 0 && frameTime != 0 && !ResourceFactory.RENDER_PERFORMANCE_LOG_ENABLED) {
				fps = frameCounter * 1000000000L / frameTime;
				frameCounter = 0;
				frameTime = 0;
			}
			if (ResourceFactory.RENDER_PERFORMANCE_LOG_ENABLED && frameCounter % 1000 == 0 && frameTime != 0) {
				fps = frameCounter * 1000000000L / frameTime;
				frameCounter = 0;
				frameTime = 0;
				System.out.println("Total frame time: \t\t" + benchmark[0]);
				System.out.println("Time in fetching sprites: \t" + benchmark[1]);
				System.out.println("Time in rendering sprites: \t" + benchmark[2]);
				System.out.println("Time in rendering level: \t" + benchmark[6]);
				System.out.println("Time in Display update: \t\t" + benchmark[3]);
				System.out.println("Time in GUI update: \t\t" + benchmark[4]);
				System.out.println("Time in GUI clear state: \t" + benchmark[5]);
				System.out.println("Time in clearing screen \t" + benchmark[7]);
				Arrays.fill(benchmark, 0);
			}
			benchmark[0] += delta;
			delta = currentFrameTime - lastFrameTime;
			frameCounter ++;
			frameTime += delta;
			lastFrameTime = currentFrameTime;
		}
	}

	@Override
	public void setGame(Game game) {
		this.mGame = game;
		 mGame.windowReady();
	}
	
	private void clearResources() {
		Display.destroy();
		AL.destroy();
	}

}
