package org.ntnu.pythia.engine.lwjgl;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import org.lwjgl.BufferUtils;
import org.lwjgl.openal.AL10;
import org.lwjgl.util.WaveData;
import org.newdawn.slick.openal.OggData;
import org.newdawn.slick.openal.OggDecoder;
import org.newdawn.slick.openal.OggInputStream;
import org.ntnu.pythia.game.SoundObject;

public class LWJGLSoundObject extends SoundObject {
	IntBuffer buffer = BufferUtils.createIntBuffer(1);
	IntBuffer source = BufferUtils.createIntBuffer(1);
	/** Position of the source sound. */
	FloatBuffer sourcePos = (FloatBuffer)BufferUtils.createFloatBuffer(3).put(new float[] { 0.0f, 0.0f, 0.0f }).rewind();
	/** Velocity of the source sound. */
	FloatBuffer sourceVel = (FloatBuffer)BufferUtils.createFloatBuffer(3).put(new float[] { 0.0f, 0.0f, 0.0f }).rewind();

	public LWJGLSoundObject(String ref) {
		super(ref);
		AL10.alGenBuffers(buffer);
		
		if (ref.toLowerCase().endsWith(".ogg")) {
			throw new RuntimeException("OGG not supported");
			/*try {
				InputStream in = LWJGLSoundObject.class.getResourceAsStream("/"+ref);
				OggInputStream oggFile = new OggInputStream(in);
				
				ByteBuffer tmp = BufferUtils.createByteBuffer(oggFile.getLength());
				byte a;
				while ((a = (byte)oggFile.read()) >= 0) tmp.put(a);
				
				tmp.rewind();
				AL10.alBufferData(buffer.get(0), AL10.AL_FORMAT_STEREO16, tmp, oggFile.getRate());
			
				//oggFile.close();
		
			} catch (IOException e) {
				e.printStackTrace();
			}*/
		} else if (ref.toLowerCase().endsWith(".wav")) {
		
			WaveData waveFile = WaveData.create(ref);
			AL10.alBufferData(buffer.get(0), waveFile.format, waveFile.data, waveFile.samplerate);
			waveFile.dispose();
		} else {
			throw new RuntimeException("Uknown file format: " + ref);
		}
		
		AL10.alGenSources(source);
		
		check(ref);
		
		AL10.alSourcei(source.get(0), AL10.AL_BUFFER,   buffer.get(0) );
		AL10.alSourcef(source.get(0), AL10.AL_PITCH,    1.0f          );
		AL10.alSourcef(source.get(0), AL10.AL_GAIN,     1.0f          );
		AL10.alSource (source.get(0), AL10.AL_POSITION, sourcePos     );
		AL10.alSource (source.get(0), AL10.AL_VELOCITY, sourceVel     );
		
		check(ref);
	}
	
	private void check(String ref) {
		int error;
		if ((error = AL10.alGetError()) != AL10.AL_NO_ERROR) {
			System.err.println("Something wrong construction sound  from " + ref + ". Error code: " + error);
		}
	}
	
	@Override
	public void play() {
		AL10.alSourcePlay(source.get(0));
	}
	
	@Override
	public void pause() {
		AL10.alSourcePause(source.get(0));
	}
	
	@Override
	public void stop() {
		AL10.alSourceStop(source.get(0));
	}
	
	@Override
	protected void finalize() {
		AL10.alDeleteBuffers(buffer);
		AL10.alDeleteSources(source);
	}

	@Override
	public boolean isPlaying() {
		return AL10.alGetSourcei(source.get(0), AL10.AL_SOURCE_STATE) == AL10.AL_PLAYING;
	}
}
