package org.ntnu.pythia.engine.lwjgl;

import java.io.IOException;

import org.lwjgl.opengl.GL11;
import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.game.RenderObject;

public class LWJGLSprite implements RenderObject {
	private Texture texture;
	
	public LWJGLSprite(LWJGLGameWindow window, String ref) {
		try {
			texture = window.getTextureLoader().getTexture(ref);
			if (texture == null)
				System.err.println("Unable to load texture: " + ref);
		} catch (IOException e) {
			System.err.println("Unable to load texture: " + ref);
			if(App.options.isDebug())
				e.printStackTrace();
		}
	}

	@Override
	public int getWidth() {
		return texture.getImageWidth();
	}

	@Override
	public int getHeight() {
		return texture.getImageHeight();
	}

	@Override
	public void draw(double x, double y, double rot, int width, int height) {
		
		GL11.glPushMatrix();
		
		GL11.glTranslated(x, y, 0);
		GL11.glRotatef((float)rot, 0, 0, 1);
			texture.bind();
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST );
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST );
			
			GL11.glColor3f(1, 1, 1);
			GL11.glBegin(GL11.GL_QUADS);
			{
				GL11.glTexCoord2f(0, 0);
				GL11.glVertex2f(- width / 2, - height / 2);
				GL11.glTexCoord2f(0, texture.getTextureHeight());
				GL11.glVertex2f(- width / 2, height / 2);
				GL11.glTexCoord2f(texture.getTextureWidth(), texture.getTextureHeight());
				GL11.glVertex2f(width / 2, height / 2);
				GL11.glTexCoord2f(texture.getTextureWidth(), 0);
				GL11.glVertex2f(width / 2, - height / 2);
			}
			GL11.glEnd();
		
		GL11.glPopMatrix();
	}

	@Override
	public void pause() {
	}

	@Override
	public void resume() {
	}

	@Override
	public void drawShadow(double x, double y, double rot, int width,
			int height, int z_index) {
		GL11.glPushMatrix();
		
		GL11.glTranslated(x - width / 2, y - height / 2, 0);
		GL11.glRotatef((float)rot, 0, 0, 1);
			texture.bind();
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST );
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST );
			
			GL11.glColor4f(0, 0, 0, 0.3f);
			for (int i = 1; i <= z_index * 3; i++) {
				GL11.glBegin(GL11.GL_QUADS);
				{
					int dx = i * 2, dy = i; 
					GL11.glTexCoord2f(0, 0);
					GL11.glVertex2f(dx, dy);
					GL11.glTexCoord2f(0, texture.getTextureHeight());
					GL11.glVertex2f(dx, height + dy);
					GL11.glTexCoord2f(texture.getTextureWidth(), texture.getTextureHeight());
					GL11.glVertex2f(width + dx, height + dy);
					GL11.glTexCoord2f(texture.getTextureWidth(), 0);
					GL11.glVertex2f(width + dx, dy);
				}
				GL11.glEnd();
			}
		
		GL11.glPopMatrix();
	}

	@Override
	public void drawDamage(double x, double y, double rot, int width,
			int height, float damage) {
		GL11.glPushMatrix();
		
		GL11.glTranslated(x - width / 2, y - height / 2, 0);
		GL11.glRotatef((float)rot, 0, 0, 1);
			texture.bind();
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST );
			GL11.glTexParameteri( GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST );
			
			GL11.glColor4f(1f, 0, 0, 1f * damage/100f);
				GL11.glBegin(GL11.GL_QUADS);
				{
					GL11.glTexCoord2f(0, 0);
					GL11.glVertex2f(0, 0);
					GL11.glTexCoord2f(0, texture.getTextureHeight());
					GL11.glVertex2f(0, height);
					GL11.glTexCoord2f(texture.getTextureWidth(), texture.getTextureHeight());
					GL11.glVertex2f(width, height);
					GL11.glTexCoord2f(texture.getTextureWidth(), 0);
					GL11.glVertex2f(width, 0);
				}
				GL11.glEnd();
		
		GL11.glPopMatrix();
	}
}
