package org.ntnu.pythia.engine.lwjgl;

import org.lwjgl.opengl.GL11;

public class Texture {
	/** The GL target type */
	private int target;
	/** The GL texture ID */
	private int textureID;
	/** The height of the image */
	private int height;
	/** The width of the image */
	private int width;
	/** The height of the texture */
	private int t_height;
	/** The width of the texture */
	private int t_width;
	/** The ratio of the width of the image to the width of the texture */
	private float widthRatio;
	/** The ratio of the height of the image to the width of the texture */
	private float heightRatio;
	
	/**
	 * Create a new texture
	 * @param target The GL target
	 * @param textureID The GL texture ID
	 */
	public Texture(int target, int textureID) {
		this.target = target;
		this.textureID = textureID;
	}
	
	/**
	 * Bind GL context to the texture
	 */
	public void bind() {
		GL11.glBindTexture(target, textureID);
	}
	
	public void setImageHeight(int height) {
		this.height = height;
		setHeight();
	}
	
	public void setImageWidth(int width) {
		this.width = width;
		setWidth();
	}
	
	public int getImageHeight() {
		return height;
	}
	
	public int getImageWidth() {
		return width;
	}

	public float getTextureHeight() {
		return heightRatio;
	}

	public float getTextureWidth() {
		return widthRatio;
	}
	
	public void setTextureHeight(int t_height) {
		this.t_height = t_height;
		setHeight();
	}
	
	public void setTextureWidth(int t_width) {
		this.t_width = t_width;
		setWidth();
	}
	
	private void setHeight() {
		if (t_height != 0)
			heightRatio = ((float) height) / t_height;
	}
	
	private void setWidth() {
		if (t_width != 0)
			widthRatio = ((float) width) / t_width;
	}
	
	@Override
	protected void finalize() {
		GL11.glDeleteTextures(textureID);
	}
}
