package org.ntnu.pythia.game;

public class AnimationObject implements RenderObject {
	private final RenderObject[] frames;
	
	private int frameCounter = 0;
	private double timeElapsed;
	private final double length;
	private final double frameStep;
	private boolean paused = false;
	private boolean playonce = false;
	
	public AnimationObject(RenderObject[] frames, double length) {
		this.frames = frames;
		this.length = length;
		this.frameStep = length / (1.0 * frames.length);
	}
	
	public void animate(double delta) {
		timeElapsed += delta;
		while (!paused && timeElapsed > frameStep) {
			timeElapsed -= frameStep;
			frameCounter++;
			if (frameCounter == frames.length) {
				if (playonce) { paused = true; frameCounter--; }
				else frameCounter = 0;
			}
		}
	}
	
	public void setPlayonce(boolean playonce) {
		this.playonce = playonce;
	}
	
	public void pause() {
		paused = true;
	}
	
	public void resume() {
		paused = false;
	}
	
	public double getLength() {
		return length;
	}

	@Override
	public int getWidth() {
		return frames[frameCounter].getWidth();
	}

	@Override
	public int getHeight() {
		return frames[frameCounter].getHeight();
	}

	@Override
	public void draw(double x, double y, double rotation, int width, int height) {
		frames[frameCounter].draw(x, y, rotation, width, height);
	}

	@Override
	public void drawShadow(double posX, double posY, double rot, int width,
			int height, int z_index) {
		frames[frameCounter].drawShadow(posX, posY, rot, width, height, z_index);
	}

	@Override
	public void drawDamage(double posX, double posY, double rot, int width,
			int height, float damage) {
		frames[frameCounter].drawDamage(posX, posY, rot, width, height, damage);
	}	
}
