package org.ntnu.pythia.game;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import org.ntnu.pythia.game.level.LevelCell;
import org.ntnu.pythia.game.level.LevelLayout;

public class LevelObject {
	
	private LevelLayout mLevel;
	
	private GameObject[] tiles;
	
	private final int x_size;
	private final int y_size;
	
	private final static int TILE_SIZE = 32;
	
	public LevelObject(LevelLayout level) {
		this.mLevel = level;
		int x_size = mLevel.getXSize();
		int y_size = mLevel.getYSize();
		this.x_size = x_size;
		this.y_size = y_size;
		
		tiles = new GameObject[x_size * y_size];
		
		for (int y = 0; y < y_size; y++) {
			for (int x = 0; x < x_size; x++) {
				int posX = x * TILE_SIZE + TILE_SIZE / 2;
				int posY = y * TILE_SIZE + TILE_SIZE / 2;
				LevelCell cell = mLevel.getCellAt(x, y);
								
				GameObject newTile;
				if(cell.isSolid())
					newTile = new WallTile(cell.getTextureRef(), posX, posY);
				else
					newTile = new FloorTile(cell.getTextureRef(), posX, posY);
				
				if (cell.isHurtful()) {
					newTile.setHarmful(true, cell.getDamagePerSecond(), true);
					newTile.setCollisionObject(new CollisionObject(32, 32, 0, 0, newTile, false, false));
				}
				if (cell.isHollow()) {
					newTile.setHarmful(true, 1, true);
					newTile.setCollisionObject(new CollisionObject(32, 32, 0, 0, newTile, false, false, true));
				}
				
				tiles[x_size * y + x] = newTile; 
			}
		}
	}
	
	public GameObject[] getStaticObjects() {
		return tiles;
	}
	
	
	public List<GameObject> getDrawingTiles(Rectangle view) {
		int start_x = (view.x - TILE_SIZE) / TILE_SIZE;
		int end_x = (view.x + view.width + TILE_SIZE) / TILE_SIZE ;
		int start_y = (view.y - TILE_SIZE) / TILE_SIZE;
		int end_y = (view.y + view.width + TILE_SIZE) / TILE_SIZE ;
		start_x = (start_x < 0) ? 0 : start_x;
		end_x = (end_x > x_size) ? x_size : end_x;
		start_y = (start_y < 0) ? 0 : start_y;
		end_y = (end_y > y_size) ? y_size : end_y;
		List<GameObject> result = new ArrayList<GameObject>();
		for (int y = start_y; y < end_y; y++)
			for (int x = start_x; x < end_x; x++)
				result.add(tiles[x_size * y + x]);
		return result;
	}
}
