package org.ntnu.pythia.game;

import java.util.HashMap;
import java.util.Map;

import org.ntnu.pythia.engine.ResourceFactory;

public class PlayerObject extends GameObject {
	
	private SoundObject collisionSound, step_left, step_right, deathSound, hurtSound;
	private Map<String, AnimationObject> animations = new HashMap<String, AnimationObject>(); 
	
	/**
	 * Constructs a new player in x and y coordinates
	 * @param x The x coordinate of the player
	 * @param y The y coordinate of the player
	 */
	public PlayerObject(int x, int y) {
		super("player", x, y, 1, 44, 32);
		setCollisionObject(new CollisionObject(24, 24, 0, 4, this, true));
		collisionSound = ResourceFactory.getSoundObject("sounds/pang1.wav");
		step_left = ResourceFactory.getSoundObject("step_left");
		step_right = ResourceFactory.getSoundObject("step_right");
		deathSound = ResourceFactory.getSoundObject("die");
		hurtSound = ResourceFactory.getSoundObject("hurt");
		animations.put("e", ResourceFactory.getAnimationObject("player_e"));
		animations.put("w", ResourceFactory.getAnimationObject("player_w"));
		animations.put("s", ResourceFactory.getAnimationObject("player_s"));
		animations.put("n", ResourceFactory.getAnimationObject("player_n"));
		animations.put("ne", ResourceFactory.getAnimationObject("player_ne"));
		animations.put("nw", ResourceFactory.getAnimationObject("player_nw"));
		animations.put("se", ResourceFactory.getAnimationObject("player_se"));
		animations.put("sw", ResourceFactory.getAnimationObject("player_sw"));
		currentSprite = animations.get("s");
		this.vLimit = 200;
		this.a = 200;
		setMortal(true, 100f, 0.5f, false, "player_die");
	}
	
	private double chatDelay = 0;
	private double stepCounter = 0;
	private boolean step_left_played = false;
	private boolean step_right_played = false;
	@Override
	public void move(double delta) {
		super.move(delta);
		if (this.falling) return;
		chatDelay += delta;
		if (currentSprite != null && !isDead())
			if (Math.abs(vX)+ Math.abs(vY) > 2) {
					if (step_left != null && step_right != null) {
					stepCounter += delta;
					if (stepCounter < 0.25 && step_left_played == false) {
						step_left.play();
						step_left_played = true;
					} else if (stepCounter > 0.25 && stepCounter < 0.5 && step_right_played == false) {
						step_right.play();
						step_right_played = true;
					} else if (stepCounter > 0.5) {
						stepCounter = 0;
						step_left_played = false;
						step_right_played = false;
					}
				}
				currentSprite.resume();
				nextFrame();
			} else currentSprite.pause();
		if (chatDelay > 1.0) {
			chatDelay = 0;
			//System.out.printf("Player's health: %f\n", getHealth());
			/*if (gameWorld != null)
				gameWorld.say(this, "Give me some cake");*/
		}
	}
	
	private void nextFrame() {
		double angle = Math.atan2(-vY, vX);
		if (angle > Math.PI * 7.0/8.0 || angle < - Math.PI * 7.0/8.0) // left
			currentSprite = animations.get("w");
		else if (angle < Math.PI * 7.0/8.0 && angle > Math.PI * 5.0/8.0) // up-left
			currentSprite = animations.get("nw");
		else if (angle < Math.PI * 5.0/8.0 && angle > Math.PI * 3.0/8.0) // up
			currentSprite = animations.get("n");
		else if (angle < Math.PI * 3.0/8.0 && angle > Math.PI * 1.0/8.0) // up-right
			currentSprite = animations.get("ne");
		else if (angle < Math.PI * 1.0/8.0 && angle > -Math.PI * 1.0/8.0) // right
			currentSprite = animations.get("e");
		else if (angle < -Math.PI * 1.0/8.0 && angle > -Math.PI * 3.0/8.0) // down-right
			currentSprite = animations.get("se");
		else if (angle < -Math.PI * 3.0/8.0 && angle > -Math.PI * 5.0/8.0) // down
			currentSprite = animations.get("s");
		else if (angle < -Math.PI * 5.0/8.0 && angle > -Math.PI * 7.0/8.0) // down-left
			currentSprite = animations.get("sw");
	}
	
	@Override
	protected void die() {
		super.die();
		deathSound.play();
	}
	
	@Override
	protected void hurting(double damage) {
		super.hurting(damage);
		if (!hurtSound.isPlaying())
			hurtSound.play();
	}
}
