package org.ntnu.pythia.game.behaviour;

import java.awt.Color;
import java.util.List;

import org.ntnu.pythia.game.level.ManipulateableObject;
import org.ntnu.pythia.game.level.xml.Bind.Binding;
import org.ntnu.pythia.python.ListListener;
import org.ntnu.pythia.python.VariableListener;
import org.python.core.PyObject;

/**
 * A Manipulateable Object with an additional behaviour defined in python.
 * Will have a python script that defines how it reacts to various BehaviourEvents.
 * @author andreas
 *
 */
public class BehavedObject extends ManipulateableObject{
	
	public BehavedObject(String name, Color color, int xpos, int ypos, int z_index,
			int width, int height) {
		super(name, color, xpos, ypos, z_index, height, width);
		
	}

	public BehavedObject(String name, String appearance, int x_position,
			int y_position, int z_index, int width, int height) {
		super(name, appearance, x_position, y_position, z_index, height, width);
	}

	public BehavedObject(String name, int x_position, int y_position,
			int z_index, int width, int height) {
		super(name, x_position, y_position, z_index, height, width);
	}

	protected Behaviour behaviour;
	
	protected List<BehaviourProperty> props;
	
	/**
	 * Causes the behaviour to react according to its programming.
	 * @param event
	 * @return
	 */
	public boolean fireEvent(String event) {
		if(behaviour != null){
			return behaviour.react(event, this);
		}
		return false;
	}
	
	@Override
	public void move(double delta) {
		if (behaviour != null)
			behaviour.timeElapsed(delta, this);
		super.move(delta);
	}
	
	@Override
	public void bindVariableToAttribute(String variable, Binding bind, String defaultVal){
		if(bind.equals(Binding.Custom)){
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					BehavedObject.this.behaviour.react(newValue.toString(), BehavedObject.this);
					BehavedObject.this.behaviour.varSet(name, newValue);
				}
			});
			if(defaultVal != null)
				parser.exec(variable + " = " + defaultVal);
		}else if(bind.equals(Binding.CustomList)){
			parser.addListener(variable, new ListListener() {
				@Override
				public void variableSet(String name, PyObject index, PyObject newValue) {
					BehavedObject.this.behaviour.listSet(parser.eval(name), index.asIndex(), newValue);
				}
			});
		}else
			super.bindVariableToAttribute(variable, bind, defaultVal);
	}
	
	@Override
	public void talkTo(){
		if(!behaviour.react("talk", this))
			super.talkTo();
	}
	
	/**
	 * Causes the behaviour to react according to its programming.
	 * @param event
	 * @return
	 */
	public void react(String what){
		behaviour.react(what, this);
	}
	
	/**
	 * Get a property for this object set in the XML.
	 * @param name
	 * @return
	 */
	public String getProperty(String name){
		if(props == null || props.isEmpty())
			return null;
		for(BehaviourProperty p : props)
			if(p.name.equals(name))
				return p.value;
		return null;
	}

	public void setBehaviour(Behaviour behaviour) {
		this.behaviour = behaviour;
	}

	public void setProperties(List<BehaviourProperty> props) {
		this.props = props;
	}

	public void setProperty(String name, String value) {
		if(props != null){
			BehaviourProperty p = new BehaviourProperty();
			p.name = name;
			p.value = value;
			props.add(p);
		}
	}
}
