package org.ntnu.pythia.game.behaviour;

import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.game.level.Appearance;
import org.ntnu.pythia.game.level.ManipulateableObjectFactory;
import org.ntnu.pythia.game.level.Appearance.RenderMethod;
import org.ntnu.pythia.python.Module;
import org.python.core.PyObject;

@XmlRootElement(name = "BehavedObject")
public class BehavedObjectFactory extends ManipulateableObjectFactory {

	@XmlElement(name = "behaviour")
	protected String behaviourName;

	@XmlElement(name = "property")
	protected List<BehaviourProperty> props;

	protected Behaviour behaviour;
	
	@Override
	protected void construct() {
		try{
			if(appearance == null)
				product = new BehavedObject(name, x_position, y_position, z_index, width, height);
			else if(appearance.getRendering().getMethod() == RenderMethod.TEXTURE){ 
				product = new BehavedObject(name, appearance.getAppearance(), x_position, y_position, z_index, width, height);
			}else{
				Appearance.FillColor fc = (Appearance.FillColor) appearance.getRendering();
				product = new BehavedObject(name, fc.getColor(), x_position, y_position, z_index, width, height);
				product.setColor(fc.getColor());
			}
		}catch (RuntimeException e) {
			throw new RuntimeException("Error in rendering definition for object '" + name + "'.", e);
		}
	}

	@Override
	protected void init(){
		super.init();
		if(behaviourName != null){
			Module m = App.globals.getModule(behaviourName);
			parser.exec("from org.ntnu.pythia.game.behaviour import Behaviour");
			m.load(parser);
			PyObject behaviourClass = parser.get(m.getClassName());
			if(behaviourClass== null)
				throw new RuntimeException(m.getClassName() + " not found.");
			behaviour = (Behaviour) behaviourClass.__call__().__tojava__(Behaviour.class);
			((BehavedObject)product).setBehaviour(behaviour);
			behaviour.setParent((BehavedObject)product);
		}
		
	}
	
	@Override
	protected void setValues(){
		super.setValues();
		if(props != null)
			((BehavedObject)product).setProperties(props);
	}
}
