package org.ntnu.pythia.game.behaviour;

import org.ntnu.pythia.game.GameObject;
import org.python.core.PyObject;

/** 
 * This is used to determine the actions performed by a game object such as an enemy. 
 * Also used as mini controllers for showing changes on screen.
 * @author andreas
 *
 */

public interface Behaviour {

	/** 
	 * Make the owner of the behaviour react to an event.
	 * @param event
	 * @return true if this Behaviour handles the event.
	 */
	public boolean react(String event, BehavedObject bo);
	
	/**
	 * Inform the script that certain amount of time has elapsed
	 * @param sec
	 */
	public void timeElapsed(double delta, BehavedObject go);
	
	/**
	 * Give the script access to the parent.
	 * @param bo
	 */
	public void setParent(BehavedObject bo);

	/**
	 * Works in coordination with custom-list binding. Fires when a list is changed.
	 * @param list
	 * @param asIndex
	 * @param newValue
	 */
	public void listSet(PyObject list, int asIndex, PyObject newValue);

	/**
	 * Works in coordination with a custom binding. Fires when a variable is changed.
	 * @param var
	 * @param newValue
	 */
	public void varSet(String var, PyObject newValue);
}
