package org.ntnu.pythia.game.behaviour;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.game.level.AbstractObjectFactory;
import org.ntnu.pythia.game.level.ManipulateableObjectFactory;
import org.ntnu.pythia.game.level.SimpleObject;
/**
 * Creates a {@link CompositeObject} from XML.
 * @author andreas
 *
 */
@XmlRootElement(name = "CompositeObject")
public class CompositeObjectFactory extends BehavedObjectFactory{
	@XmlAttribute(name = "cwidth")
	int cellsWide;
	@XmlAttribute(name = "cheight")
	int cellsHigh;
	
	@XmlElementRef
	List<ManipulateableObjectFactory> templates = new ArrayList<ManipulateableObjectFactory>();
	
	Map<String, AbstractObjectFactory> templatesByName = new HashMap<String, AbstractObjectFactory>();
	
	@Override
	protected void construct() {
		try{
			product = new CompositeObject(name, x_position, y_position, z_index, height, width);
		
		}catch (RuntimeException e) {
			throw new RuntimeException("Error while constructing composite object '" + name + "'.", e);
		}
	}

	@Override
	protected void init(){
		for(ManipulateableObjectFactory fact : templates){
			fact.setParser(parser);
			templatesByName.put(fact.getName(), fact);
		}
		super.init();
	}
	
	@Override
	protected void setValues(){
		super.setValues();
		if(cellsWide != 0)
			((CompositeObject)product).setCellsWide(cellsWide);
		if(cellsHigh != 0)
			((CompositeObject)product).setCellsHigh(cellsHigh);
		if(width != 0 && cellsWide != 0)
			((CompositeObject)product).setCellWidth(this.width / cellsWide);
		if(height != 0 && cellsHigh != 0)
			((CompositeObject)product).setCellHeight(this.height / cellsHigh);
		if(!templates.isEmpty()){
			((CompositeObject)product).setTemplates(templatesByName);
			SimpleObject[] cells = new SimpleObject[cellsWide * cellsHigh];
			templates.get(0).setParser(parser);
			for (int i = 0; i < cells.length; i++) {
				cells[i] = templates.get(0).getObject();
			}
			((CompositeObject)product).setCells(cells);
		}
		
	}
}
