package org.ntnu.pythia.game.behaviour;

import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.game.level.xml.GlobalsStoreableObject;

/**
 * Defines simple interaction for a ManipulateableObject. 
 * Interaction is what happens when a player walks up to the object and presses enter.
 * It has states which decide what an object will 'say' when a player 'talks' to it.
 * @author andreas
 *
 */
@XmlRootElement(name = "Conversation")
public class Interaction implements GlobalsStoreableObject{
	
	@XmlAttribute
	private String id;
	
	@XmlAttribute
	private String src;
	
	@XmlElement(name = "state")
	private List<ConversationState> states;
	
	private String state = "initial";
	
	public String getPath(){
		return getPathForState(state);
	}
	
	public String getPathForState(String state){
		for(ConversationState s : states){
			if(s.getId().equals(state)){
				return s.getContentPath();
			}
		}
		throw new RuntimeException("No content found for state '" + state + "'.");
	}
	
	public String getStatusMsgForState() {
		for(ConversationState s : states){
			if(s.getId().equals(state)){
				return s.getStatusMsg();
			}
		}
		return null;
	}
	
	public void setState(String state){
		this.state = state;
	}

	@Override
	public String getId() {
		return id;
	}

	@Override
	public String getSource() {
		return src;
	}

	@Override
	public boolean needsLoading() {
		return (states == null || states.isEmpty());
	}
/*
	public static void main(String...args){
		Interaction i = new Interaction();
		i.src = "/convotest.xml";
		i = Globals.finalize(i, i.getClass());
		System.out.println(i.getPathForState("initial"));
		
	}
	*/

	public String getState() {
		return state;
	}
}
