package org.ntnu.pythia.game.level;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

import org.ntnu.pythia.engine.ResourceFactory;
import org.ntnu.pythia.game.CollisionModelFactory;
import org.ntnu.pythia.game.RenderObject;
import org.ntnu.pythia.game.behaviour.Interaction;
import org.ntnu.pythia.game.level.xml.CollisionScheme;
import org.ntnu.pythia.python.PythiaParser;
/**
 * Basis for creating a factory for creating objects from XML. (Abstract Factory)
 * @author andreas
 */
public abstract class AbstractObjectFactory {

	/** How this object appears on screen. */
	@XmlElement(name = "appearance")
	protected Appearance appearance;

	@XmlAttribute(name = "x", required = true)
	protected int x_position;
	@XmlAttribute(name = "y", required = true)
	protected int y_position;
	@XmlAttribute(name = "z_index")
	protected int z_index = 1;
	@XmlAttribute(name = "height", required = true)
	protected int height;
	@XmlAttribute(name = "width", required = true)
	protected int width;
	
	@XmlAttribute
	public String ref;
	
	@XmlAttribute(name="extends")
	public String extend;
	
	@XmlElement(name = "name", required = true)
	protected String name;

	@XmlElement(name = "Conversation")
	protected Interaction interaction; 

	@XmlElement(name = "initialSolid")
	protected boolean isolid;
	
	@XmlElement(name = "CollisionScheme", required = true)
	protected CollisionScheme cScheme;
	
	@XmlElement(name = "harmful")
	protected Harmful harmful = null;
	@XmlElement(name = "mortal")
	protected Mortal mortal = null;
	
	protected SimpleObject product;

	protected PythiaParser parser;
	
	public void setParser(PythiaParser parser) {
		this.parser = parser;
	}

	/*
	 * XXX: Meant to be used to make behaviour shared by several objects.
	 * How should we implement this? 
	 */
	@XmlElement(name = "type")
	protected String type; //currently not in use
	
	/**
	 * Makes sure the object is good to be fetched.
	 */
	protected void init() {
		product.setCollisionObject(CollisionModelFactory.createCModel(product));
	}
	
	/**
	 * Assigns values from the XML to the object
	 */
	protected void setValues(){
		if(z_index != 1)
			product.setZindex(z_index);
		if(appearance != null)
			product.setAppearance(appearance);
		
		if(interaction != null)
			product.setInteraction(interaction); 
		
		if(!isolid)
			product.setSolid(isolid);
		if(cScheme != null)
			product.setCollisionScheme(cScheme);
		
		if (harmful != null)
			this.product.setHarmful(true, harmful.damage, harmful.areal);
		if (mortal != null) {
			String death_anim = null;
			if (mortal.appearance != null)
				death_anim = mortal.appearance.getAppearance();
			this.product.setMortal(true, mortal.health, mortal.reg_rate, mortal.die_on_collision, death_anim);
		}
		
		product.setParser(parser);
	}
	
	/**
	 * Runs the actual constructor for the object. Does not return anything.
	 */
	protected abstract void construct();
		
	/**
	 * Returns the Object
	 * @return
	 */
	public abstract SimpleObject getObject();

	/**
	 * Returns an object that extends a different object. 
	 * @param extend
	 * @return
	 */
	public SimpleObject getObject(AbstractObjectFactory extend){
		if(extend == null)
			throw new RuntimeException("Unknown reference to extend: '"+ this.extend + "'.");
		extend.construct();
		this.product = extend.product;
		extend.setValues();
		this.setValues();
		this.init();
		return this.product;
	}
	
	public String getName(){
		return name;
	}

	public void setY(int y) {
		this.y_position = y;
	}
	
	public void setX(int x) {
		this.x_position = x;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setWidth(int width) {
		this.width = width;
	}
	
	protected static class Harmful {
		@XmlAttribute(name="damage")
		public int damage = 0;
		@XmlAttribute(name="areal")
		public boolean areal = false;
	}
	
	protected static class Mortal {
		@XmlAttribute(name="health")
		public int health = 0;
		@XmlAttribute(name="reg_rate")
		public int reg_rate = 0;
		@XmlAttribute(name="die_on_collision")
		public boolean die_on_collision = false;
		@XmlElement
		public Appearance appearance = null;
	}
}
