package org.ntnu.pythia.game.level;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ntnu.pythia.engine.GameWorld;
import org.ntnu.pythia.engine.ResourceFactory;
import org.ntnu.pythia.game.behaviour.Interaction;
import org.ntnu.pythia.game.level.xml.LevelObjectWrapper;
import org.ntnu.pythia.python.Module;
import org.ntnu.pythia.python.PythiaParser;
import org.ntnu.pythia.python.unit.Test;

/**
 * Model for a level. 
 * @author andreas
 *
 */

public class Level {

	private int levelNumber;

	private String levelName;
	
	private LevelLayout layout;
	
	private LevelObjectWrapper levelObjects; 

	private Interaction interaction; 
	
	/** The players spawn location x coordinate */
	private int spawnx;
	
	/** The players spawn location y coordinate */
	private int spawny;
	
	/** The tests to be cleared before the player can leave the level */
	private List<Test> tests;

	private PythiaParser masterParser;
	
	private GameWorld gameWorld;
	
	private Map<String, SimpleObject> objectsByName = new HashMap<String, SimpleObject>();

	private LoadableResource graphics;
	
	private LoadableResource sounds;
	
	public int getLevelNumber() {
		return levelNumber;
	}
	
	public String getLevelName() {
		return levelName;
	}
		
	public SimpleObject getObject(String name){
		return objectsByName.get(name);
	}
	
	public LevelLayout getLayout(){
		return layout;
	}
		
	public Collection<SimpleObject> getObjects(){
		return objectsByName.values();
	}
	
	public void doWelcomeMessage(){
		gameWorld.messageBox(ResourceFactory.getTextString(interaction.getPath()));
	}
	
	public int getSpawnPositionX() {
		return (int) spawnx;
	}
	
	public int getSpawnPositionY() {
		return (int) spawny;
	}
	
	public List<Test> getTests(){
		return tests;
	}
	
	public void addTest(Test t){
		tests.add(t);
	}

	public void setGameWorld(GameWorld gameWorld) {
		this.gameWorld = gameWorld;
	}

	public void setLevelNumber(int levelNumber) {
		this.levelNumber = levelNumber;
	}

	public void setLevelLayout(LevelLayout layout) {
		this.layout = layout;
	}

	public void addObject(String name, SimpleObject obj) {
		objectsByName.put(obj.getName(), obj);
	}

	public void setLevelNumber(String levelName) {
		this.levelName = levelName;
	}

	public void setWelcomeMessage(Interaction interaction) {
		this.interaction = interaction;
	}

	public void setSpawny(int spawny) {
		this.spawny = spawny;
	}

	public void setSpawnx(int spawnx) {
		this.spawnx = spawnx;
	}

	public void setTests(List<Test> tests) {
		this.tests = tests;
	}

	public void setLevelGraphics(LoadableResource graphics) {
		this.graphics = graphics;
	}

	public LoadableResource getLevelGraphics() {
		return graphics;
	}
	
	public void setLevelSounds(LoadableResource sounds) {
		this.sounds = sounds;
	}
	
	public LoadableResource getLevelSounds() {
		return sounds;
	}

}
