package org.ntnu.pythia.game.level;

import java.util.HashMap;
import java.util.Map;

import org.ntnu.pythia.game.level.xml.TileDefinition;
import org.ntnu.pythia.game.level.xml.TileDefinition.TileRef;
/**
 * a cell on the background grid of a level. Could be moved into another class eventually.
 * @author andreas
 *
 */
public class LevelCell {
	
	private char code;
	
	private static Map<Character, TileRef> tilesByChar; 
	
	public LevelCell(char c) {
		this.code = c;
		if(!tilesByChar.containsKey(c))
			throw new RuntimeException("Tile definition does not contain definition for char '" + c + "'.");
	}
	
	public static void setDef(TileDefinition tiledef){
		tilesByChar = new HashMap<Character, TileDefinition.TileRef>();
		for(TileRef ref : tiledef.getAllTileRefs()){
			tilesByChar.put(ref.getChar(), ref);
		}
	}

	/**
	 * @deprecated Use getTextureRef and other methods directly instead 
	 * of caring what this tile is represented as in the layout.
	 * @return
	 */
	@Deprecated
	public char getCode(){
		return code;
	}
	
	public String getTextureRef(){
		return tilesByChar.get(new Character(code)).id;
	}
	
	public boolean isSolid(){
		return tilesByChar.get(new Character(code)).solid;
	}
	
	public boolean isHollow(){
		return tilesByChar.get(new Character(code)).hollow;
	}
	
	public boolean isHurtful(){
		return tilesByChar.get(new Character(code)).hurtfull != 0;
	}
	
	public int getDamagePerSecond() {
		return tilesByChar.get(new Character(code)).hurtfull;
	}
	
	public boolean isWater(){
		return tilesByChar.get(new Character(code)).water;
	}
}
