package org.ntnu.pythia.game.level;

import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.engine.ResourceFactory;

@XmlRootElement(name = "LevelGraphics")
public class LevelGraphics implements LoadableResource {
	
	@XmlElementRef
	private List<GraphicRef> graphicRefs;
	
	public void loadResource() {
		if (graphicRefs != null)
			for(GraphicRef g : graphicRefs)
				g.load();
	}
	
	public static abstract class GraphicRef{
		@XmlAttribute(name = "id", required = true)
		public String ref;
		abstract void load();
	}
	
	@XmlRootElement(name = "texture")
	public static class TextureRef extends GraphicRef{
		@XmlAttribute(name = "src", required = true)
		public String src;
		void load(){
			System.out.println("Loading " + ref);
			ResourceFactory.getRenderObject(ref, src);
		}
	}
	
	@XmlRootElement(name = "animation")
	public static class AnimationRef extends GraphicRef{
		@XmlAttribute(name = "length", required = true)
		public double length;
		@XmlElementRef
		private List<TextureRef> frames;
		
		void load(){
			System.out.println("Loading " + ref);
			ResourceFactory.getAnimationObject(ref, getReferences(), getFilenames(), length);
		}
		public String[] getFilenames() {
			String[] filenames = new String[frames.size()];
			for (int i = 0; i < filenames.length; i++)
				filenames[i] = frames.get(i).src;
			return filenames;
		}
		public String[] getReferences() {
			String[] references = new String[frames.size()];
			for (int i = 0; i < references.length; i++)
				references[i] = frames.get(i).ref;
			return references;
		}
	}

}
