package org.ntnu.pythia.game.level;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.game.level.xml.TileDefinition;

/**
 * The Layout of a level. Including walls and floors and gaps.
 * Not including enemies, consoles and player locations.
 * @author andreas
 *
 */
public class LevelLayout {

	@XmlAttribute(name = "size_x", required = true)
	private int size_x;
	@XmlAttribute(name = "size_y", required = true)
	private int size_y;
	
	private LevelCell[] cells;
	
	@XmlElement(required = true)
	private String src;

	@XmlElement
	TileDefinition tileDef;
	
	/**
	 * Load the layout defined in the xml into memory
	 * @throws IOException
	 */
	public void loadLayout() throws IOException{
		if(tileDef == null)
			throw new RuntimeException("No tile definition for layout at '" +src+ "' was defined in level.xml.");
		LevelCell.setDef(tileDef);
		
		InputStream is;
		if(App.options.isExternal())
			is = new FileInputStream(new File(App.options.getExternal(), src));
		else
			is = getClass().getResourceAsStream("/levels/" + src);
		if(is == null)
			if(App.options.isExternal())
				throw new RuntimeException("Error opening file: '" + App.options.getExternal() + "/" + src + "'.");
			else
				throw new RuntimeException("Error opening file: '" + "/levels/" + src + "'.");
		Scanner scanner = new Scanner(is);
		List<LevelCell> cellList = new LinkedList<LevelCell>();
		
		while (scanner.hasNextLine()) {
			String line = scanner.nextLine();
			for (int x = 1; x <= line.length(); x++) {
				cellList.add(new LevelCell(line.charAt(x-1)));
			}
		}
		
		is.close();
		
		cells = cellList.toArray(new LevelCell[0]);
	}
	
	public LevelCell getCellAt(int x, int y){
		//TODO: check for out of range.
		return cells[x + y*size_x];
	}
	
	public String getSource(){
		return src;
	}
	
	public int getXSize(){
		return size_x;
	}
	
	public int getYSize(){
		return size_y;
	}
}
