package org.ntnu.pythia.game.level;

import java.awt.Color;

import org.ntnu.pythia.game.level.Appearance.ColorChoice;
import org.ntnu.pythia.game.level.xml.Bind.Binding;
import org.ntnu.pythia.python.VariableListener;
import org.python.core.PyObject;

/**
 * Will respond to changes to variables defined in its bindings.
 * @author andreas
 *
 */

public class ManipulateableObject extends SimpleObject{

	public ManipulateableObject(String name, String ref, int xpos, int ypos, int z_index,
			int height, int width) {
		super(name, ref, xpos, ypos, z_index, height, width);
	}

	public ManipulateableObject(String name, Color color, int xpos, int ypos, int z_index,
			int height, int width) {
		super(name, color, xpos, ypos, z_index, height, width);
	}
	
	public ManipulateableObject(String name, int x_position, int y_position,
			int z_index, int height, int width) {
		super(name, x_position, y_position, z_index, height, width);
	}

	/**
	 * Bind a python variable to an attribute of this (or the inner) object.
	 */
	public void bindVariableToAttribute(String variable, Binding bind, String defaultVal){
		switch (bind) {
		case Xpos:
			parser.set(variable, this.getX());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setX(Integer.parseInt(newValue.toString()));
				}
			});
			break;
		case Ypos:
			parser.set(variable, this.getY());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setY(Integer.parseInt(newValue.toString()));
				}
			});
			break;
		case Color:
			ColorChoice cc = Appearance.ColorChoice.byColor(this.getColor());
			if(cc!= null)
				parser.exec(variable +"='" +cc.toString().toLowerCase()+"'");
			else
				parser.exec(variable +"='black'");
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ColorChoice cc = Appearance.ColorChoice.byName(newValue.toString());
					if(cc== null)
						cc=ColorChoice.BLACK;
					ManipulateableObject.this.setColor(cc.getColor());
				}
			});
			break;
		case Custom:
			throw new RuntimeException("Custom binding only available for Behaved objects");
		case ConvoState:
			parser.set(variable, this.getConvoState());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setConvoState(newValue.toString());
				}
			});
			break;
		case Solid:
			parser.set(variable, this.isSolid());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setSolid("true".equalsIgnoreCase(newValue.toString()));
				}
			});
			break;
		case Xv:
			parser.set(variable, this.getVx());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setVx(Integer.parseInt(newValue.toString()));
				}
			});
			break;
		case Yv:
			parser.set(variable, this.getVy());
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.setVy(Integer.parseInt(newValue.toString()));
				}
			});
			break;
		case MoveRight:
			parser.set(variable, this.moveRight);
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.moveRight = (Boolean.parseBoolean(newValue.toString()));
				}
			});
			break;
		case MoveLeft:
			parser.set(variable, this.moveLeft);
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.moveLeft = (Boolean.parseBoolean(newValue.toString()));
				}
			});
			break;
		case MoveUp:
			parser.set(variable, this.moveUp);
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.moveUp = (Boolean.parseBoolean(newValue.toString()));
				}
			});
			break;
		case MoveDown:
			parser.set(variable, this.moveDown);
			parser.addListener(variable, new VariableListener(){
				@Override
				public void variableSet(String name, PyObject newValue) {
					ManipulateableObject.this.moveDown = (Boolean.parseBoolean(newValue.toString()));
				}
			});
			break;
		default:
			break;
		}
	}
}