package org.ntnu.pythia.game.level;

import java.util.LinkedList;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.game.level.Appearance.RenderMethod;
import org.ntnu.pythia.game.level.xml.Bind;

@XmlRootElement(name = "DynamicObject")
public class ManipulateableObjectFactory extends AbstractObjectFactory {

	@XmlElement(name = "bind")
	private List<Bind> binds = new LinkedList<Bind>();

	@Override
	protected void construct() {
		try{
			if(appearance.getRendering().method == RenderMethod.TEXTURE){ 
				product = new ManipulateableObject(name, appearance.getAppearance(), x_position, y_position, z_index, height, width);
			}else{
				Appearance.FillColor fc = (Appearance.FillColor) appearance.getRendering();
				product = new ManipulateableObject(name, fc.getColor(), x_position, y_position, z_index, height, width);
				product.setColor(fc.getColor());
			}
		}catch (RuntimeException e) {
			throw new RuntimeException("Error in rendering definition for object '" + name + "'.", e);
		}
	}

	@Override
	protected void setValues(){
		super.setValues();
		if(binds != null)
			bindAll();
	}
	
	@Override
	public SimpleObject getObject() {
		construct();
		setValues();
		init();
		return product;
	}

	/**
	 * Bind or re-bind all bindings for this object.
	 */
	protected void bindAll(){
		for(Bind b : binds)
			b.perform((ManipulateableObject) product);
	}

}
