package org.ntnu.pythia.game.level;

import java.awt.Color;

import org.ntnu.pythia.engine.ResourceFactory;
import org.ntnu.pythia.game.CollisionModelFactory;
import org.ntnu.pythia.game.GameObject;
import org.ntnu.pythia.game.behaviour.Interaction;
import org.ntnu.pythia.game.level.Appearance.RenderMethod;
import org.ntnu.pythia.game.level.xml.CollisionScheme;
import org.ntnu.pythia.python.PythiaParser;

public abstract class SimpleObject extends GameObject {

	public SimpleObject(String name, Color color, int xpos, int ypos, int z_index,
			int height, int width) {
		super(name, xpos, ypos, z_index, width, height, color);
	}

	public SimpleObject(String name, String ref, int xpos, int ypos, int z_index,
			int height, int width) {
		super(name, xpos, ypos, z_index, width, height, ref);
	}

	public SimpleObject(String name, int x_position, int y_position,
			int z_index, int height, int width) {
		super(name, x_position, y_position, z_index, width, height);
	}

	protected Interaction interaction; 

	protected Appearance appearance;
	
	protected CollisionScheme cScheme; 

	/** The parser that will execute the python code that will change this object. */
	protected PythiaParser parser;
	
	public PythiaParser getParser() {
		return parser;
	}
	
	public void setSolid(boolean solid){
		if(solid)
			setCollisionObject(CollisionModelFactory.createCModel(this));
		else
			setCollisionObject(null);
	}

	public void setParser(PythiaParser parser){
		this.parser = parser;
	}
	
	public void changeRendering(String name){
		appearance.setState(name);
		try{
			if(appearance.getRendering().method == RenderMethod.TEXTURE){
				Appearance.Texture t = (Appearance.Texture) appearance.getRendering();
				setAppearance(t.id);
			}else{
				Appearance.FillColor fc = (Appearance.FillColor) appearance.getRendering();
				setAppearance(fc.getColor(), width, height);
			}
		}catch (RuntimeException e) {
			throw new RuntimeException("Error in rendering definition for object '" + getName() + "' with name '" + getName() + "'.", e);
		}
	}

	/**
	 * Trigger the talking mechanism on this object.
	 */
	public void talkTo(){
		//Open new message window
		if(interaction != null){
			messageBox(ResourceFactory.getTextString(interaction.getPath()));
		}else{
			System.out.println(this + " was talked to but had no interaction");
		}
	}

	public String getConvoState(){
		if(interaction != null)
			return interaction.getState();
		else return "none";
	}

	public void setConvoState(String state){
		if(interaction != null)
			interaction.setState(state);
	}
	

	public int getHeight() {
		return height;
	}

	public int getWidth() {
		return width;
	}
	
	public void remove(){
		ResourceFactory.getGameWorld().removeGameObject(this);
	}

	public CollisionScheme getCScheme() {
		return cScheme;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	void setAppearance(Appearance appearance) {
		this.appearance = appearance;
	}

	public Appearance getAppearanceObject() {
		return appearance;
	}
	
	void setInteraction(Interaction interaction) {
		this.interaction = interaction;
	}

	public void setCollisionScheme(CollisionScheme cScheme) {
		this.cScheme = cScheme;
	}	
	
	public String getCloseMessage() {
		if (interaction == null) return null;
		String msg = interaction.getStatusMsgForState();
		if (msg == null) msg = "Press ENTER to interact";
		return msg;
	}

	@Override
	public String toString(){
		return "<" + getClass().getSimpleName() + " : " + getName() + ">";
	}
}
