package org.ntnu.pythia.game.level.xml;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.game.level.ManipulateableObject;

/**
 * Object built from xml to record the binding between a variable and 
 * given attributes in a ManipulateableObject.
 * @author andreas
 *
 */
@XmlRootElement
public class Bind {

	/**	Which attribute of this object a variable should be bound to. */
	@XmlEnum
	public enum Binding{
		/** Bind the variable to the x-position of the object. */
		@XmlEnumValue(value = "x-pos")
		Xpos,
		/** Bind the variable to the y-position of the object. */
		@XmlEnumValue(value = "y-pos")
		Ypos, 
		/** Bind the variable to the colour of the object. */
		@XmlEnumValue(value = "color")
		Color,
		/** Bind the variable to the Conversation state of the object. */
		@XmlEnumValue(value = "conversationState")
		ConvoState,
		/** Bind the variable to the velocity of the object in x-direction. */
		@XmlEnumValue(value = "x-velocity")
		Xv,
		/** Bind the variable to the velocity of the object in y-direction. */
		@XmlEnumValue(value = "y-velocity")
		Yv,
		/** Bind the variable to the solidity ({@link ManipulateableObject.isSolid()}) of the object. */
		@XmlEnumValue(value = "solidity")
		Solid,
		/** Bind the variable to a custom listener in python. */
		@XmlEnumValue(value = "custom")
		Custom,
		@XmlEnumValue(value = "customlist")
		CustomList,
		MoveRight,
		MoveLeft,
		MoveUp,
		MoveDown
	}
	
	@XmlAttribute(name = "variable", required = true)
	String variableName;
	
	@XmlAttribute(name = "binding", required = true)
	Binding binding;
	
	@XmlAttribute
	String defaultVal;

	/**
	 * Make this binding happen.
	 * @param target
	 */
	public void perform(ManipulateableObject target){
		target.bindVariableToAttribute(variableName, binding, defaultVal);
		
	}
	
}
