package org.ntnu.pythia.game.player;

import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlList;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.python.Module;

/**
 * Object for storing stats related to the player. 
 * @author andreas
 *
 */
@XmlRootElement
public class Player {
	@XmlAttribute(name="name")
	private String playerSlotName;
	/* 
	 * XXX: Suggestion for fields
	 * - keys will be identified as a string, a given string unlocks 
	 * a given door. Cannot have multiple keys
	 * - Programming keywords might not be feasible to implement
	 * - Powerups?
	 * - Items?
	 * - Program size should be simple to implement, might not be that 
	 * interesting 
	 */
	/** The keys (for doors etc) that this character is holding */
	@XmlElement
	@XmlList
	List<String> keys = new LinkedList<String>();
	
	/** Items that the player has */ 
	@XmlElement
	@XmlList
	List<String> items;//redundant?
	
//	List<Object> powerups;//XXX: currently not in use?
	
	@XmlElementRef
	List<Module> programs;
	
	/** The maximum program size the player is allowed to make */
	@XmlElement
	int allowedCharacters;
	
	/** The level the player is currently at. (And will load onto the next time the game starts) */
	@XmlElement(name ="level")
	private String levelName;
	
	public boolean hasKey(String key){
		if(keys == null)
			return false;
		return keys.contains(key);
	}
	
	public void giveKey(String key){
		keys.add(key);
	}

	public void setCurrentLevelName(String name) {
		levelName = name;
	}
	
	public String getCurrentLevelName() {
		return levelName;
	}

	public void setName(String name) {
		this.playerSlotName = name;
	}
	
	public String getName() {
		return playerSlotName;
	}

	public void prepareToSave() {
		// nothing to prepare?
	}
	
	public void init() {
	}

	public boolean hasLaptop(){
		return keys.contains("laptop");
	}
	
	public boolean hasEditor(){
		return keys.contains("editor");
	}
	
	public void saveScript(Module script) {
		if(programs == null)
			programs = new LinkedList<Module>();
		if(!programs.contains(script))
			programs.add(script);
	}

	public Module getScript(String name) {
		if(programs == null)
			return null;
		for(Module m : programs)
			if(m.getName().equals(name))
				return m;
		return null;
	}
}
