package org.ntnu.pythia.python;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.python.core.PyDictionary;
import org.python.core.PyFunction;
import org.python.core.PyList;
import org.python.core.PyMethod;
import org.python.core.PyObject;
import org.python.core.PyStringMap;

public class ListenableVariableMap extends PyStringMap implements VariableListener{
	
	private Map<String, List<VariableListener>> listenerMap = new HashMap<String, List<VariableListener>>(); 
	private Map<String, List<ListListener>> listListenerMap = new HashMap<String, List<ListListener>>();
	private Map<String, List<FunctionListener>> functionMap = new HashMap<String, List<FunctionListener>>();
	
	protected void addListener(String variableName, VariableListener listener){
//		System.out.printf("added listener for %s: %s\n", variableName, listener);
		if(listenerMap.containsKey(variableName))
			listenerMap.get(variableName).add(listener);
		else{
			List<VariableListener> list = new LinkedList<VariableListener>();
			list.add(listener);
			listenerMap.put(variableName, list);
		}
	}
	
	protected void addListener(String variableName, FunctionListener listener){
//		System.out.printf("added listener for %s: %s\n", variableName, listener);
		if(functionMap.containsKey(variableName))
			functionMap.get(variableName).add(listener);
		else{
			List<FunctionListener> list = new LinkedList<FunctionListener>();
			list.add(listener);
			functionMap.put(variableName, list);
		}
	}
		
	@Override
	public void __setitem__(String key, PyObject value) {
		if(listenerMap.containsKey(key))
			variableSet(key, value);
		if(value instanceof PyList ){
			ListenableList l = new ListenableList((PyList) value, this, key);
			l.addListener(listListenerMap.get(key));
			super.__setitem__(key, l);
		}else if ( value instanceof PyDictionary){
			ListenableDictionary l = new ListenableDictionary((PyDictionary) value, this, key);
			l.addListener(listListenerMap.get(key));
			super.__setitem__(key, l);
		}else if(value instanceof PyMethod || value instanceof PyFunction){
			if(functionMap.containsKey(key))
				methodSet(key, value);
			else
				super.__setitem__(key, value);
		}
		else
			super.__setitem__(key, value);
	}

	private void methodSet(String name, PyObject value) {
		if(functionMap.containsKey(name))
			for(FunctionListener l : functionMap.get(name)){
				l.functionSet(name, value);
				if(!l.shouldOverwrite(name, value))
					return;
			}
		super.__setitem__(name, value);
	}

	public void variableSet(String name, PyObject value) {
		if(listenerMap.containsKey(name))
			for(VariableListener l : listenerMap.get(name))
				l.variableSet(name, value);
	}

	protected void addListener(String variableName, ListListener listener) {
		if(listListenerMap.containsKey(variableName))
			listListenerMap.get(variableName).add(listener);
		else{
			List<ListListener> list = new LinkedList<ListListener>();
			list.add(listener);
			listListenerMap.put(variableName, list);
		}
	}
}
