package org.ntnu.pythia.python;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.engine.ResourceFactory;

/** A python Module to be used anywhere in Pythia */
@XmlRootElement(name = "Module")
public class Module {
	String description;
	String name;
	String className;
	String src;

	/** Load this module directly into the parser */
	public void load(PythiaParser p){
		File f;
		if(App.options.isExternal())
			f = new File(App.options.getExternal(), src);
		else
			f= new File(ResourceFactory.CURRENT_FOLDER, src);
		try {
			InputStream is;
			if (f.exists())
				is  = new FileInputStream(f);
			else is = getClass().getResourceAsStream(src);
			if(is == null)
				throw new RuntimeException("Could not find source for module '" + name + "' at location: " + src);
			p.execfile(is, "<Pythia module " + name + ">");
			is.close();
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Could not find source for module '" + name + "' at location: " + f.getPath(), e);
		} catch (IOException e) {
			throw new RuntimeException("Error while reading source for module '" + name + "' at location: " + f.getPath(), e);
		}
	}

	public String getDescription() {
		return description;
	}

	@XmlAttribute
	public void setDescription(String description) {
		this.description = description;
	}

	public String getClassName() {
		return className;
	}

	public String getName() {
		return name;
	}

	@XmlElement
	public void setName(String name) {
		this.name = name;
	}

	public String getSrc() {
		return src;
	}

	@XmlElement
	public void setSrc(String src) {
		this.src = src;
	}

	@XmlElement
	public void setClassName(String className) {
		this.className = className;
	}

	public String getContentAsString() throws IOException {
		File f = new File(ResourceFactory.CURRENT_FOLDER, src);
		StringBuilder sb = new StringBuilder();
		FileReader fr = new FileReader(f);
		while(fr.ready())
			sb.append((char) fr.read());

		return sb.toString();
	}
}


