package org.ntnu.pythia.python;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.engine.ResourceFactory;
import org.python.core.Py;
import org.python.core.PySystemState;
import org.python.util.PythonInterpreter;

/**
 * Parser for all Pythia python code. Has the functionality to add listeners for certain variables.
 * @author andreas
 *
 */
public class PythiaParser extends PythonInterpreter{

	private GlobalTracer globalTracer;

	public PythiaParser(){
		super();
		setLocals(new ListenableVariableMap());
		setOut(App.output);
		globalTracer = new GlobalTracer();
		//		setErr(App.output);
	}

	private PythiaParser(PySystemState state){
		super(null, state);
		setLocals(new ListenableVariableMap());
		setOut(App.output);
	}

	public void exec(String s, boolean interruptable) throws InterruptedException {
		if(s.trim().equals("System.exit()")){
			if(App.globals.playerHasKey("secret")){
				exec("import time");
				ResourceFactory.getGameWorld().showConsole();
				exec("print 'System is shutting down...'");
				exec("time.sleep(0.8)");
				exec("print 'releasing resources'");
				exec("time.sleep(0.8)");
				exec("print 'freeing memory'");
				exec("time.sleep(0.8)");
				exec("print 'I give up. you win.'");
				exec("time.sleep(0.8)");
				exec("print 'fading to black'");
				exec("time.sleep(0.8)");
				//Anything before we quit?
				System.exit(0);
			}
		} else {

			Py.getThreadState().tracefunc = globalTracer;
			super.exec(s);
		}
	}

	/**
	 * Get global tracer
	 * Used to kill things
	 * @return a global tracer of class GlobalTracer
	 */
	public void interruptThread() {
		globalTracer.interrupt();
	}

	/**
	 * Creates another parser that shares namespace with this one.
	 * @param state the (different) PySystemState for the new parser.
	 * @return
	 */
	public PythiaParser spawnNewParserWithSameNamespace(PySystemState state){
		PythiaParser ret = new PythiaParser(state);
		ret.setLocals(this.getLocals());
		return ret;
	}

	/**
	 * Add a listener to trigger when a variable is set. 
	 * @param variableName
	 * @param listener
	 */
	public void addListener(String variableName, VariableListener listener){
		((ListenableVariableMap) getLocals()).addListener(variableName, listener);
	}

	/**
	 * Add a listener to trigger when a list is changed. 
	 * @param variableName
	 * @param listener
	 */
	public void addListener(String variableName, ListListener listener){
		((ListenableVariableMap) getLocals()).addListener(variableName, listener);
	}

	/**
	 * Add a listener to trigger when a function is changed. 
	 * @param functionName
	 * @param listener
	 */
	public void addListener(String functionName, FunctionListener listener){
		((ListenableVariableMap) getLocals()).addListener(functionName, listener);
	}

	/**
	 * Removes all variables, loaded modules, and listeners.
	 */
	public void clear() {
		setLocals(new ListenableVariableMap());
		App.globals.loadLibs();
	}
}