package org.ntnu.pythia.python;

import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.python.core.Py;
import org.python.core.PySystemState;

import org.ntnu.pythia.game.level.ManipulateableObject;
import org.ntnu.pythia.game.level.xml.Bind;

/**
 * Model for a Python script. Allows for separate threads, with separate variable listeners.
 * Not really used for anything. 
 * @author andreas 
 *
 */
@XmlRootElement
public class Script {

	@XmlElement(name = "stepTime", required = false, defaultValue = "0")
	int stepTime; //run a step every x ms

	/** Script name */
	@XmlElement(name = "name", required = true)
	String name;

	/** The source for the script */
	@XmlElement(name = "src", required = true)
	String src;

	@XmlElement(name = "bind")
	private List<Bind> binds = new LinkedList<Bind>();

	@XmlElement(name = "debug")
	private boolean runAsDebug = true;

	/** The parser this script executes on */
	PythiaParser parser;

	/** The target this script affects */
	private ManipulateableObject target;
	
	private ScriptTracer trace; 

	public void prepare(){
		PySystemState.initialize();
		trace = new ScriptTracer(stepTime);
	    Py.getThreadState().tracefunc = trace;
	    
		InputStream is = getClass().getResourceAsStream(src);
		if(is == null)
			throw new RuntimeException("Could not find source for script '" + name + "' at location: " + src);
		
		trace.setCode(getClass().getResource(src));
		parser.execfile(is);
		parser.exec("init()");
	}

	public void setTarget(ManipulateableObject tar){
		target = tar;
	}

	public void setBinds(Bind bind){
		binds.add(bind);
	}

	public void setScriptName(String name){
		this.name = name;
	}

	public List<Bind> getBinds(){
		return binds;
	}

	/**
	 * Bind or re-bind all bindings for this script.
	 */
	public void bindAll(){
		target.setParser(parser);
		for(Bind b : binds)
			b.perform(target);
	}

	public void run() {
		parser.exec("exec()");
	}
/*
	public static void main(String...strings) throws JAXBException{

		PythonInterpreter p = new PythonInterpreter();

		JAXBContext context = JAXBContext.newInstance(Script.class, ManipulateableObject.class, Bind.class);
		Unmarshaller unmarshaller = context.createUnmarshaller();
		Script s = (Script)unmarshaller.unmarshal(Script.class.getResourceAsStream("/script.xml"));
		s.prepare();
//		s.run();
	}
*/
	public void setParser(PythiaParser parser) {
		this.parser = parser;
	}

	public void pause() {
		trace.halt();
	}
	
	public void unpause() {
		trace.unhalt();
	}
	
}
