package org.ntnu.pythia.python;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import org.python.core.Py;
import org.python.core.PyException;
import org.python.core.PyFrame;
import org.python.core.PyObject;
import org.python.core.TraceFunction;

//TODO: remove debug printouts
/**
 * A tracer that allows debugging of python code inside a UserScript.
 */
public class ScriptTracer extends TraceFunction {
	
	boolean halted;
	boolean step;
	boolean stop;
	long stepTime;
	URL codeBase;
	UserScript script;
	
	PyFrame activeFrame = null;
	
	public ScriptTracer(long stepTime){
		this.stepTime = stepTime;
	}
	
	public ScriptTracer(long stepTime, UserScript script){
		this.stepTime = stepTime;
		this.script = script;
	}

	public void makeTraceCurrent(PyFrame frame) {
		frame.tracefunc = this;
	}

	public void halt(){
		halted = true;
	}
	
	public void unhalt(){
		halted = false;
	}
	
	@Override
	public TraceFunction traceCall(PyFrame frame) {
		if(script != null) script.execute(frame.f_lineno);
		sleep();
		step = false;
		if (stop) interrupt();
		return this;
	}

	@Override
	public TraceFunction traceException(PyFrame frame, PyException exc) {
		activeFrame = frame;
		if(script != null) script.exception(frame.f_lineno, exc);
		sleep();
		step = false;
		return this;
	}

	@Override
	public TraceFunction traceLine(PyFrame frame, int line) {
		activeFrame = frame;
		if(script != null) script.execute(line);
		sleep();
		step = false;
		if (stop) interrupt();
		return this;
	}
	
	@Override
	public TraceFunction traceReturn(PyFrame frame, PyObject ret) {
		activeFrame = frame;
		if(script != null) script.execute(frame.f_lineno);
		sleep();
		step = false;
		if (stop) interrupt();
		return this;
	}

	public void setCode(URL url) {
		codeBase = url;
	}

	private String getLine(int line) throws IOException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(codeBase.openStream()));
		String lines = "";
		while (--line > 0) lines = reader.readLine();
		return lines;
	}

	public void step() {
		step = true;
	}

	public void stop() {
		stop = true;
	}
	
	public void stopped() {
		stop = false;
	}
	
	private void sleep() {
		try{
			while (halted && !step && !stop)
				Thread.sleep(stepTime);
		} catch (InterruptedException e) {}
	}
	
	private void interrupt() {
		throw new PyException(Py.KeyboardInterrupt, "Interrupted by loser!" );
	}

	public PyFrame getFrame() {
		return activeFrame;
	}
}
