package org.ntnu.pythia.python;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.engine.ResourceFactory;
import org.python.core.Py;
import org.python.core.PyException;
import org.python.core.PyFrame;
import org.python.core.PySystemState;


/**
 * Class for handling a program created by a user and edited in a <code>ListeningEditor</code>.
 * @author andreas
 *
 */
public class UserScript implements Runnable {

	private ScriptTracer tracer;
	private PythiaParser parser;
	private String program;
	private ListeningEditor editor;
	private Module desc;

	/**
	 * 
	 * @param playerName The name of the player. (for save locations)
	 * @param parser The parser to execute the script on.
	 * @param editor The editor which contains the script.
	 * @param programName The name of the program. (for save locations and debugging)
	 */
	public UserScript( PythiaParser parser, ListeningEditor editor){
		String name = App.globals.getPlayerName()+"_"+App.globals.getCurrentLevel().getLevelName();
		this.editor = editor;
		if(App.globals.getPlayerScript(name)==null){
			desc = new Module();
			desc.name = App.globals.getPlayerName()+"_"+App.globals.getCurrentLevel().getLevelName();
			desc.src = "/playerscripts/" + desc.name + ".py";
//			save();
		}else{
			desc = App.globals.getPlayerScript(name);
			String temp = null;
			try{
			temp = desc.getContentAsString();
			}catch (IOException e) {
				save();//file does not exist or cannot be read. attempting to make new file.
			}
			if(temp != null){
				program = temp;
				editor.setProgram(program);
			}
		}
		//		tracer = new ScriptTracer(0, this);
		//		PySystemState newState = new PySystemState();
		//		Py.getThreadState(newState).tracefunc = tracer; 
		//		this.parser = new PythiaParser(newState);
		//		PySystemState.initialize();

		// FIXME: Enabling tracing will stop all pythin code
		//	    Py.getThreadState().tracefunc = tracer;
	}

	@Override
	public void run() {
		tracer = new ScriptTracer(200, this);
		PySystemState newState = new PySystemState();
		this.parser = App.globals.getParser().spawnNewParserWithSameNamespace(newState);
		PySystemState.initialize();
		Py.getThreadState(newState).tracefunc = tracer;

		save();
		program = editor.getProgram();
		ByteArrayInputStream is = new ByteArrayInputStream(program.toString().getBytes());
		try{
			parser.execfile(is, "<user program - " + desc.name + ">");
		}catch (PyException e) {
			int lNumber = 0; //e.traceback.tb_lineno;
			String message = e.getMessage();
			editor.error(message, lNumber, e);
		} finally {
			tracer.stopped();
			editor.finished();
		}
		//done
	}

	protected void execute(int lineNumber) {
		editor.executing(lineNumber);
	}

	protected void exception(int lineNumber, PyException exc) {
		editor.error(exc.getMessage(), lineNumber, exc);
	}

	/**
	 * Halt execution.
	 */
	public void halt(){
		tracer.halt();
	}

	/**
	 * Restart execution.
	 */
	public void unhalt(){
		tracer.unhalt();
	}

	/**
	 * Take one step. (While halted)
	 */
	public void step(){
		tracer.step();
	}

	/**
	 * Set the time each execution step takes. 
	 * @param steptime the time in milliseconds.
	 */
	public void setStepTime(long steptime){
		tracer.stepTime = steptime;
	}

	public void stop() {
		tracer.stop();
	}

	/**
	 * Saves the script in the default location.
	 */
	public void save(){
		program = editor.getProgram();
		App.globals.savePlayerScript(desc);
		savePy();
	}

	/** Writes this module to disk to the location in src */
	public void savePy(){
		try {
			File f = new File(ResourceFactory.CURRENT_FOLDER, desc.src);
			f.getParentFile().mkdirs();
			if(f.createNewFile())
				System.out.println("Created new file at " + f.getAbsolutePath());
			FileWriter fw = new FileWriter(f);
			fw.write(editor.getProgram());
			fw.flush();
			fw.close();
		} catch (IOException e) {
			throw new RuntimeException("Error while saving "+ desc.name + " to file.", e);
		}
	}
	
	public PyFrame getFrame() {
		return tracer.getFrame();
	}
}
