package org.ntnu.pythia.python.unit;

import java.util.LinkedList;
import java.util.List;

import org.ntnu.pythia.engine.App;
import org.ntnu.pythia.engine.Game;
import org.ntnu.pythia.game.level.Level;

/**
 * Controller class for running tests.
 * @author andreas
 *
 */
public class Tester {

	private List<Test> tests;
	private Level currentLevel;
	private Game game;
		
	private static final Tester singleton = new Tester();
	
	private Tester() {	
	}

	public static Tester getTester(){
		return singleton;
	}
	
	public void setGame(Game game) {
		this.game = game;	
	}

	/**
	 * Load the tests from a given level.	
	 * @param level
	 */
	public void loadTests(Level level){
		this.tests = level.getTests();
		this.currentLevel = level;
		initAll();
	}

	/**
	 * Initialize all the tests. (Will be done upon loading the level)
	 */
	public void initAll(){
		if(tests != null)
			for(Test t : tests)
				t.init(App.globals.getParser());
	}

	/**
	 * Runs all the tests given for this level
	 * @return true if all the tests cleared.
	 */
	public boolean testAll(){
		if(tests != null){
			for(Test t : tests)
				if(!t.isSolved(currentLevel))
					if(t instanceof ImpliciteObjective)
						continue;
					else
						return false;
		}
		return true;
	}

	/**
	 * Gives the hints for the tests that have not passed.
	 * @return
	 */
	public List<String> getRemainingObjectives(){
		List<String> ret = new LinkedList<String>();
		if(tests != null){
			for(Test t : tests)
				if(t instanceof ImpliciteObjective)
					ret.add(t.getHint());
				else if(!t.isSolved(currentLevel))
					ret.add(t.getHint());
		}
		return ret;
	}

	/*
	 * XXX: consider moving all of this to GameWorld
	 */
	/**	
	 * Shows a summary of remaining objectives in a window in the game.
	 */
	public void showRemainingObjectives(){
		StringBuilder sb = new StringBuilder();
		List<String> objs = getRemainingObjectives();
		if(tests == null)
			sb.append("No objectives or tests for this level.");
		else if(objs.isEmpty())
			sb.append("All objectives complete. You may leave the level.");
		else{ 
			sb.append("Objectives:\n" +
			"<ol>\n");
			for(String s : objs)
				sb.append("<li>"+ s + "</li>\n");
			sb.append("</ol>\n" +
			"When you have completed this, you may proceed to the next level.");
		}
		game.showMessageBox(sb.toString());
	}

}

