package org.ntnu.other;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import junit.framework.TestCase;

public class CollectionPerformanceTest extends TestCase {
	// Linked list drops the balls on magnitude over
	private final int length = 10000;
	private ArrayList<Integer> al = new ArrayList<Integer>();
	private LinkedList<Integer> ll = new LinkedList<Integer>();
	private Vector<Integer> v = new Vector<Integer>();
	private HashSet<Integer> hs = new HashSet<Integer>();
	protected void setUp() {
		for (int i = 0; i < length; i++) {
			al.add(i);
			ll.add(i);
			v.add(i);
			hs.add(i);
		}
	}
	
	public void testArrayListIterator() {
		long startTime = System.nanoTime();
		int sum = 0;
		Iterator<Integer> it = al.iterator();
		while(it.hasNext())
			sum += it.next();
		long endTime = System.nanoTime();
		System.out.println("ArrayListIterator run in:\t" + (endTime-startTime) + " ns");
	}
	
	public void testArrayList() {
		long startTime = System.nanoTime();
		int sum = 0;
		for (int i = 0; i < al.size(); i++)
			sum += al.get(i);
		long endTime = System.nanoTime();
		System.out.println("ArrayList run in:\t\t" + (endTime-startTime) + " ns");
	}
	
	public void testLinkedListIterator() {
		long startTime = System.nanoTime();
		int sum = 0;
		Iterator<Integer> it = ll.iterator();
		while(it.hasNext())
			sum += it.next();
		long endTime = System.nanoTime();
		System.out.println("LinkedListIterator run in:\t" + (endTime-startTime) + " ns");
	}
	
	public void testLinkedList() {
		long startTime = System.nanoTime();
		int sum = 0;
		for (int i = 0; i < ll.size(); i++)
			sum += ll.get(i);
		long endTime = System.nanoTime();
		System.out.println("LinkedList run in:\t\t" + (endTime-startTime) + " ns");
	}
	
	public void testVectorIterator() {
		long startTime = System.nanoTime();
		int sum = 0;
		Iterator<Integer> it = v.iterator();
		while(it.hasNext())
			sum += it.next();
		long endTime = System.nanoTime();
		System.out.println("VectorIterator run in:\t\t" + (endTime-startTime) + " ns");
	}
	
	public void testVectorList() {
		long startTime = System.nanoTime();
		int sum = 0;
		for (int i = 0; i < v.size(); i++)
			sum += v.get(i);
		long endTime = System.nanoTime();
		System.out.println("VectorList run in:\t\t" + (endTime-startTime) + " ns");
	}
	
	public void testHashSetIterator() {
		long startTime = System.nanoTime();
		int sum = 0;
		Iterator<Integer> it = hs.iterator();
		while(it.hasNext())
			sum += it.next();
		long endTime = System.nanoTime();
		System.out.println("HashSetIterator run in:\t\t" + (endTime-startTime) + " ns");
	}
}
